@@ -22,11 +22,13 @@ struct auplay_st {
 	bool run;
 	snd_pcm_t *write;
 	int16_t *sampv;
+	void *xsampv;
 	size_t sampc;
 	auplay_write_h *wh;
 	void *arg;
 	struct auplay_prm prm;
 	char *device;
+	enum aufmt aufmt;
 };
 
 
@@ -36,6 +38,7 @@ static void auplay_destructor(void *arg)
 
 	/* Wait for termination of other thread */
 	if (st->run) {
+		debug("alsa: stopping playback thread (%s)\n", st->device);
 		st->run = false;
 		(void)pthread_join(st->thread, NULL);
 	}
@@ -44,6 +47,7 @@ static void auplay_destructor(void *arg)
 		snd_pcm_close(st->write);
 
 	mem_deref(st->sampv);
+	mem_deref(st->xsampv);
 	mem_deref(st->device);
 }
 
@@ -58,14 +62,25 @@ static void *write_thread(void *arg)
 
 	while (st->run) {
 		const int samples = num_frames;
+		void *sampv;
 
 		st->wh(st->sampv, st->sampc, st->arg);
 
-		n = snd_pcm_writei(st->write, st->sampv, samples);
+		if (st->aufmt == AUFMT_S16LE) {
+			sampv = st->sampv;
+		}
+		else {
+			sampv = st->xsampv;
+			auconv_from_s16(st->aufmt, st->xsampv,
+					st->sampv, st->sampc);
+		}
+
+		n = snd_pcm_writei(st->write, sampv, samples);
+
 		if (-EPIPE == n) {
 			snd_pcm_prepare(st->write);
 
-			n = snd_pcm_writei(st->write, st->sampv, samples);
+			n = snd_pcm_writei(st->write, st->xsampv, samples);
 			if (n != samples) {
 				warning("alsa: write error: %s\n",
 					snd_strerror(n));
@@ -89,6 +104,7 @@ int alsa_play_alloc(struct auplay_st **stp, const struct auplay *ap,
 		    auplay_write_h *wh, void *arg)
 {
 	struct auplay_st *st;
+	snd_pcm_format_t pcmfmt;
 	int num_frames;
 	int err;
 
@@ -110,6 +126,7 @@ int alsa_play_alloc(struct auplay_st **stp, const struct auplay *ap,
 	st->ap  = ap;
 	st->wh  = wh;
 	st->arg = arg;
+	st->aufmt = alsa_sample_format;
 
 	st->sampc = prm->srate * prm->ch * prm->ptime / 1000;
 	num_frames = st->prm.srate * st->prm.ptime / 1000;
@@ -120,6 +137,15 @@ int alsa_play_alloc(struct auplay_st **stp, const struct auplay *ap,
 		goto out;
 	}
 
+	if (st->aufmt != AUFMT_S16LE) {
+		size_t sz = aufmt_sample_size(st->aufmt) * st->sampc;
+		st->xsampv = mem_alloc(sz, NULL);
+		if (!st->xsampv) {
+			err = ENOMEM;
+			goto out;
+		}
+	}
+
 	err = snd_pcm_open(&st->write, st->device, SND_PCM_STREAM_PLAYBACK, 0);
 	if (err < 0) {
 		warning("alsa: could not open auplay device '%s' (%s)\n",
@@ -127,7 +153,16 @@ int alsa_play_alloc(struct auplay_st **stp, const struct auplay *ap,
 		goto out;
 	}
 
-	err = alsa_reset(st->write, st->prm.srate, st->prm.ch, num_frames);
+	pcmfmt = aufmt_to_alsaformat(st->aufmt);
+	if (pcmfmt == SND_PCM_FORMAT_UNKNOWN) {
+		warning("alsa: unknown sample format '%s'\n",
+			aufmt_name(st->aufmt));
+		err = EINVAL;
+		goto out;
+	}
+
+	err = alsa_reset(st->write, st->prm.srate, st->prm.ch, num_frames,
+			 pcmfmt);
 	if (err) {
 		warning("alsa: could not reset player '%s' (%s)\n",
 			st->device, snd_strerror(err));
@@ -141,6 +176,8 @@ int alsa_play_alloc(struct auplay_st **stp, const struct auplay *ap,
 		goto out;
 	}
 
+	debug("alsa: playback started (%s)\n", st->device);
+
  out:
 	if (err)
 		mem_deref(st);
