@@ -177,7 +177,6 @@ static int open_encoder(struct videnc_state *st,
 
 	/* params to avoid libavcodec/x264 default preset error */
 	if (st->codec_id == AV_CODEC_ID_H264) {
-		st->ctx->me_method = ME_UMH;
 		st->ctx->me_range = 16;
 		st->ctx->qmin = 10;
 		st->ctx->qmax = 51;
@@ -633,7 +632,8 @@ int encode(struct videnc_state *st, bool update, const struct vidframe *frame)
 		break;
 
 	case AV_CODEC_ID_H264:
-		err = h264_packetize(st->mb, st->encprm.pktsize,
+		err = h264_packetize(st->mb->buf, st->mb->end,
+				     st->encprm.pktsize,
 				     st->pkth, st->arg);
 		break;
 
