@@ -12,6 +12,14 @@
 #include <gio/gio.h>
 #include "gtk_mod.h"
 
+#ifdef USE_LIBNOTIFY
+#include <libnotify/notify.h>
+#endif
+
+#if GLIB_CHECK_VERSION(2,40,0) || defined(USE_LIBNOTIFY)
+#define USE_NOTIFICATIONS 1
+#endif
+
 /* About */
 #define COPYRIGHT " Copyright (C) 2010 - 2015 Alfred E. Heggestad et al."
 #define COMMENTS "A modular SIP User-Agent with audio and video support"
@@ -42,9 +50,10 @@ struct gtk_mod {
 	GSList *accounts_menu_group;
 	struct dial_dialog *dial_dialog;
 	GSList *call_windows;
+	GSList *incoming_call_menus;
 };
 
-struct gtk_mod mod_obj;
+static struct gtk_mod mod_obj;
 
 enum gtk_mod_events {
 	MQ_POPUP,
@@ -57,6 +66,7 @@ enum gtk_mod_events {
 
 static void answer_activated(GSimpleAction *, GVariant *, gpointer);
 static void reject_activated(GSimpleAction *, GVariant *, gpointer);
+static void denotify_incoming_call(struct gtk_mod *, struct call *);
 
 static GActionEntry app_entries[] = {
 	{"answer", answer_activated, "x", NULL, NULL, {0} },
@@ -97,6 +107,7 @@ static void menu_on_about(GtkMenuItem *menuItem, gpointer arg)
 			NULL);
 }
 
+
 static void menu_on_quit(GtkMenuItem *menuItem, gpointer arg)
 {
 	struct gtk_mod *mod = arg;
@@ -109,6 +120,7 @@ static void menu_on_quit(GtkMenuItem *menuItem, gpointer arg)
 	info("quit from gtk\n");
 }
 
+
 static void menu_on_dial(GtkMenuItem *menuItem, gpointer arg)
 {
 	struct gtk_mod *mod = arg;
@@ -118,6 +130,7 @@ static void menu_on_dial(GtkMenuItem *menuItem, gpointer arg)
 	dial_dialog_show(mod->dial_dialog);
 }
 
+
 static void menu_on_dial_contact(GtkMenuItem *menuItem, gpointer arg)
 {
 	struct gtk_mod *mod = arg;
@@ -126,6 +139,7 @@ static void menu_on_dial_contact(GtkMenuItem *menuItem, gpointer arg)
 	mqueue_push(mod->mq, MQ_CONNECT, (char *)uri);
 }
 
+
 static void init_contacts_menu(struct gtk_mod *mod)
 {
 	struct le *le;
@@ -166,6 +180,26 @@ static void menu_on_presence_set(GtkMenuItem *item, struct gtk_mod *mod)
 }
 
 
+#ifdef USE_NOTIFICATIONS
+static void menu_on_incoming_call_answer(GtkMenuItem *menuItem,
+		struct gtk_mod *mod)
+{
+	struct call *call = g_object_get_data(G_OBJECT(menuItem), "call");
+	denotify_incoming_call(mod, call);
+	mqueue_push(mod->mq, MQ_ANSWER, call);
+}
+
+
+static void menu_on_incoming_call_reject(GtkMenuItem *menuItem,
+		struct gtk_mod *mod)
+{
+	struct call *call = g_object_get_data(G_OBJECT(menuItem), "call");
+	denotify_incoming_call(mod, call);
+	mqueue_push(mod->mq, MQ_HANGUP, call);
+}
+#endif
+
+
 static GtkMenuItem *accounts_menu_add_item(struct gtk_mod *mod,
 		struct ua *ua)
 {
@@ -192,6 +226,7 @@ static GtkMenuItem *accounts_menu_add_item(struct gtk_mod *mod,
 	return GTK_MENU_ITEM(item);
 }
 
+
 static GtkMenuItem *accounts_menu_get_item(struct gtk_mod *mod,
 		struct ua *ua)
 {
@@ -265,18 +300,26 @@ static void accounts_menu_set_status(struct gtk_mod *mod,
 }
 
 
+#ifdef USE_NOTIFICATIONS
 static void notify_incoming_call(struct gtk_mod *mod,
 		struct call *call)
 {
-	GNotification *notification;
-	GVariant *target;
+	static const char *title = "Incoming call";
+	const char *msg = call_peeruri(call);
+	GtkWidget *call_menu;
+	GtkWidget *menu_item;
+#if defined(USE_LIBNOTIFY)
+	NotifyNotification *notification;
+#endif
+
+#if GLIB_CHECK_VERSION(2,40,0)
 	char id[64];
+	GVariant *target;
+	GNotification *notification = g_notification_new(title);
 
 	re_snprintf(id, sizeof id, "incoming-call-%p", call);
 	id[sizeof id - 1] = '\0';
 
-	notification = g_notification_new("Incoming call");
-
 #if GLIB_CHECK_VERSION(2,42,0)
 	g_notification_set_priority(notification,
 			G_NOTIFICATION_PRIORITY_URGENT);
@@ -285,24 +328,80 @@ static void notify_incoming_call(struct gtk_mod *mod,
 #endif
 
 	target = g_variant_new_int64(GPOINTER_TO_INT(call));
-	g_notification_set_body(notification, call_peeruri(call));
+	g_notification_set_body(notification, msg);
 	g_notification_add_button_with_target_value(notification,
 			"Answer", "app.answer", target);
 	g_notification_add_button_with_target_value(notification,
 			"Reject", "app.reject", target);
 	g_application_send_notification(mod->app, id, notification);
 	g_object_unref(notification);
+
+#elif defined(USE_LIBNOTIFY)
+	/* If glib does not have GNotification, use libnotify instead. */
+	if (!notify_is_initted())
+		return;
+	notification = notify_notification_new(title, msg, "baresip");
+	notify_notification_set_urgency(notification, NOTIFY_URGENCY_CRITICAL);
+	notify_notification_show(notification, NULL);
+	g_object_unref(notification);
+#else
+	(void)msg;
+	(void)title;
+#endif
+
+	/* Add incoming call to the app menu */
+	call_menu = gtk_menu_new();
+	menu_item = gtk_menu_item_new_with_mnemonic("_Incoming call");
+	g_object_set_data(G_OBJECT(menu_item), "call", call);
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item),
+			call_menu);
+	gtk_menu_shell_prepend(GTK_MENU_SHELL(mod->app_menu), menu_item);
+	mod->incoming_call_menus = g_slist_append(mod->incoming_call_menus,
+			menu_item);
+
+	menu_item = gtk_menu_item_new_with_label(call_peeruri(call));
+	gtk_widget_set_sensitive(menu_item, FALSE);
+	gtk_menu_shell_append(GTK_MENU_SHELL(call_menu), menu_item);
+
+	menu_item = gtk_menu_item_new_with_mnemonic("_Accept");
+	g_object_set_data(G_OBJECT(menu_item), "call", call);
+	g_signal_connect(G_OBJECT(menu_item), "activate",
+			G_CALLBACK(menu_on_incoming_call_answer), mod);
+	gtk_menu_shell_append(GTK_MENU_SHELL(call_menu), menu_item);
+
+	menu_item = gtk_menu_item_new_with_mnemonic("_Reject");
+	g_object_set_data(G_OBJECT(menu_item), "call", call);
+	g_signal_connect(G_OBJECT(menu_item), "activate",
+			G_CALLBACK(menu_on_incoming_call_reject), mod);
+	gtk_menu_shell_append(GTK_MENU_SHELL(call_menu), menu_item);
 }
+#endif
 
 
-static void denotify_incoming_call(struct gtk_mod *mod,
-		struct call *call)
+static void denotify_incoming_call(struct gtk_mod *mod, struct call *call)
 {
+	GSList *item, *next;
+
+#if GLIB_CHECK_VERSION(2,40,0)
 	char id[64];
 
 	re_snprintf(id, sizeof id, "incoming-call-%p", call);
 	id[sizeof id - 1] = '\0';
 	g_application_withdraw_notification(mod->app, id);
+#endif
+
+	/* Remove call submenu */
+	for (item = mod->incoming_call_menus; item; item = next) {
+		GtkWidget *menu_item = item->data;
+		next = item->next;
+
+		if (call == g_object_get_data(G_OBJECT(menu_item), "call")) {
+			gtk_widget_destroy(menu_item);
+			mod->incoming_call_menus =
+				g_slist_delete_link(mod->incoming_call_menus,
+						    item);
+		}
+	}
 }
 
 
@@ -319,6 +418,7 @@ static void answer_activated(GSimpleAction *action, GVariant *parameter,
 	}
 }
 
+
 static void reject_activated(GSimpleAction *action, GVariant *parameter,
 		gpointer arg)
 {
@@ -332,6 +432,7 @@ static void reject_activated(GSimpleAction *action, GVariant *parameter,
 	}
 }
 
+
 static struct call_window *new_call_window(struct gtk_mod *mod,
 		struct call *call)
 {
@@ -369,9 +470,12 @@ static struct call_window *get_create_call_window(struct gtk_mod *mod,
 
 void gtk_mod_call_window_closed(struct gtk_mod *mod, struct call_window *win)
 {
+	if (!mod)
+		return;
 	mod->call_windows = g_slist_remove(mod->call_windows, win);
 }
 
+
 static void ua_event_handler(struct ua *ua,
 		enum ua_event ev,
 		struct call *call,
@@ -392,9 +496,11 @@ static void ua_event_handler(struct ua *ua,
 		accounts_menu_set_status(mod, ua, ev);
 		break;
 
+#ifdef USE_NOTIFICATIONS
 	case UA_EVENT_CALL_INCOMING:
 		notify_incoming_call(mod, call);
 		break;
+#endif
 
 	case UA_EVENT_CALL_CLOSED:
 		win = get_call_window(mod, call);
@@ -435,15 +541,24 @@ static void ua_event_handler(struct ua *ua,
 	gdk_threads_leave();
 }
 
+
+#ifdef USE_NOTIFICATIONS
 static void message_handler(const struct pl *peer, const struct pl *ctype,
 			    struct mbuf *body, void *arg)
 {
 	struct gtk_mod *mod = arg;
-	GNotification *notification;
 	char title[128];
 	char msg[512];
+
+#if GLIB_CHECK_VERSION(2,40,0)
+	GNotification *notification;
+#elif defined(USE_LIBNOTIFY)
+	NotifyNotification *notification;
+#endif
+
 	(void)ctype;
 
+
 	/* Display notification of chat */
 
 	re_snprintf(title, sizeof title, "Chat from %r", peer);
@@ -452,11 +567,23 @@ static void message_handler(const struct pl *peer, const struct pl *ctype,
 	re_snprintf(msg, sizeof msg, "%b",
 			mbuf_buf(body), mbuf_get_left(body));
 
+#if GLIB_CHECK_VERSION(2,40,0)
 	notification = g_notification_new(title);
 	g_notification_set_body(notification, msg);
 	g_application_send_notification(mod->app, NULL, notification);
 	g_object_unref(notification);
+
+#elif defined(USE_LIBNOTIFY)
+	(void)mod;
+
+	if (!notify_is_initted())
+		return;
+	notification = notify_notification_new(title, msg, "baresip");
+	notify_notification_show(notification, NULL);
+	g_object_unref(notification);
+#endif
 }
+#endif
 
 
 static void popup_menu(struct gtk_mod *mod, GtkMenuPositionFunc position,
@@ -491,9 +618,12 @@ static gboolean status_icon_on_button_press(GtkStatusIcon *status_icon,
 
 void gtk_mod_connect(struct gtk_mod *mod, const char *uri)
 {
+	if (!mod)
+		return;
 	mqueue_push(mod->mq, MQ_CONNECT, (char *)uri);
 }
 
+
 static void warning_dialog(const char *title, const char *fmt, ...)
 {
 	va_list ap;
@@ -589,12 +719,14 @@ static void mqueue_handler(int id, void *data, void *arg)
 	}
 }
 
+
 static void *gtk_thread(void *arg)
 {
 	struct gtk_mod *mod = arg;
 	GtkMenuShell *app_menu;
 	GtkWidget *item;
 	GError *err = NULL;
+	struct le *le;
 
 	gdk_threads_init();
 	gtk_init(0, NULL);
@@ -611,6 +743,10 @@ static void *gtk_thread(void *arg)
 		err = NULL;
 	}
 
+#ifdef USE_LIBNOTIFY
+	notify_init("baresip");
+#endif
+
 	mod->status_icon = gtk_status_icon_new_from_icon_name("call-start");
 	gtk_status_icon_set_tooltip_text (mod->status_icon, "baresip");
 
@@ -622,6 +758,7 @@ static void *gtk_thread(void *arg)
 	mod->contacts_inited = false;
 	mod->dial_dialog = NULL;
 	mod->call_windows = NULL;
+	mod->incoming_call_menus = NULL;
 
 	/* App menu */
 	mod->app_menu = gtk_menu_new();
@@ -636,7 +773,7 @@ static void *gtk_thread(void *arg)
 			mod->accounts_menu);
 
 	/* Add accounts to submenu */
-	for (struct le *le = list_head(uag_list()); le; le = le->next) {
+	for (le = list_head(uag_list()); le; le = le->next) {
 		struct ua *ua = le->data;
 		accounts_menu_add_item(mod, ua);
 	}
@@ -853,21 +990,28 @@ static int module_init(void)
 	err = mqueue_alloc(&mod_obj.mq, mqueue_handler, &mod_obj);
 	if (err)
 		return err;
+
+	aufilt_register(&vumeter);
+#ifdef USE_NOTIFICATIONS
+	err = message_init(message_handler, &mod_obj);
+	if (err)
+		return err;
+#endif
+
+	err = cmd_register(cmdv, ARRAY_SIZE(cmdv));
+	if (err)
+		return err;
+
+	/* start the thread last */
 	err = pthread_create(&mod_obj.thread, NULL, gtk_thread,
 			     &mod_obj);
 	if (err)
 		return err;
 
-	aufilt_register(&vumeter);
-	err = message_init(message_handler, &mod_obj);
-	if (err)
-		return err;
-
-	err = cmd_register(cmdv, ARRAY_SIZE(cmdv));
-
 	return err;
 }
 
+
 static int module_close(void)
 {
 	cmd_unregister(cmdv);
@@ -881,6 +1025,17 @@ static int module_close(void)
 	aufilt_unregister(&vumeter);
 	message_close();
 
+#ifdef USE_LIBNOTIFY
+	if (notify_is_initted())
+		notify_uninit();
+#endif
+
+	g_slist_free(mod_obj.accounts_menu_group);
+	g_slist_free(mod_obj.call_windows);
+	g_slist_free(mod_obj.incoming_call_menus);
+
+	uag_event_unregister(ua_event_handler);
+
 	return 0;
 }
 
