@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2011 Creytiv.com
  */
-
+#define _DEFAULT_SOURCE 1
 #define _BSD_SOURCE 1
 #include <pthread.h>
 #include <string.h>
@@ -62,7 +62,7 @@ static void *video_thread(void *arg)
 
 	while (st->run) {
 
-		(void)usleep(4000);
+		sys_msleep(4);
 
 		now = tmr_jiffies();
 
