@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2010 Creytiv.com
  */
+#define _DEFAULT_SOURCE 1
 #define _BSD_SOURCE 1
 #include <string.h>
 #include <time.h>
@@ -11,6 +12,26 @@
 #include <baresip.h>
 
 
+/**
+ * @defgroup vidloop vidloop
+ *
+ * A video-loop module for testing
+ *
+ * Simple test module that loops back the video frames from a
+ * video-source to a video-display, optionally via a video codec.
+ *
+ * Example usage without codec:
+ \verbatim
+  baresip -ev
+ \endverbatim
+ *
+ * Example usage with codec:
+ \verbatim
+  baresip -evv
+ \endverbatim
+ */
+
+
 /** Internal pixel-format */
 #ifndef VIDLOOP_INTERNAL_FMT
 #define VIDLOOP_INTERNAL_FMT (VID_FMT_YUV420P)
@@ -112,7 +133,10 @@ static int packet_handler(bool marker, const uint8_t *hdr, size_t hdr_len,
 		}
 	}
 
-	display(vl, &frame);
+	if (vidframe_isvalid(&frame)) {
+
+		display(vl, &frame);
+	}
 
  out:
 	mem_deref(mb);
