@@ -1,5 +1,5 @@
 /**
- * @file call.c  Call Control
+ * @file src/call.c  Call Control
  *
  * Copyright (C) 2010 Creytiv.com
  */
@@ -64,7 +64,9 @@ struct call {
 	time_t time_start;        /**< Time when call started               */
 	time_t time_conn;         /**< Time when call initiated             */
 	time_t time_stop;         /**< Time when call stopped               */
+	bool outgoing;
 	bool got_offer;           /**< Got SDP Offer from Peer              */
+	bool on_hold;             /**< True if call is on hold              */
 	struct mnat_sess *mnats;  /**< Media NAT session                    */
 	bool mnat_wait;           /**< Waiting for MNAT to establish        */
 	struct menc_sess *mencs;  /**< Media encryption session state       */
@@ -391,6 +393,7 @@ static void audio_error_handler(int err, const char *str, void *arg)
 }
 
 
+#ifdef USE_VIDEO
 static void video_error_handler(int err, const char *str, void *arg)
 {
 	struct call *call = arg;
@@ -401,6 +404,7 @@ static void video_error_handler(int err, const char *str, void *arg)
 	call_stream_stop(call);
 	call_event_handler(call, CALL_EVENT_CLOSED, str);
 }
+#endif
 
 
 static void menc_error_handler(int err, void *arg)
@@ -581,6 +585,8 @@ int call_connect(struct call *call, const struct pl *paddr)
 
 	info("call: connecting to '%r'..\n", paddr);
 
+	call->outgoing = true;
+
 	/* if the peer-address is a full SIP address then we need
 	 * to parse it and extract the SIP uri part.
 	 */
@@ -779,8 +785,13 @@ int call_hold(struct call *call, bool hold)
 	if (!call || !call->sess)
 		return EINVAL;
 
+	if (hold == call->on_hold)
+		return 0;
+
 	info("call: %s %s\n", hold ? "hold" : "resume", call->peer_uri);
 
+	call->on_hold = hold;
+
 	FOREACH_STREAM
 		stream_hold(le->data, hold);
 
@@ -837,6 +848,8 @@ int call_debug(struct re_printf *pf, const struct call *call)
 			  call->local_name, call->local_uri,
 			  call->peer_name, call->peer_uri,
 			  net_af2name(call->af));
+	err |= re_hprintf(pf, " direction: %s\n",
+			  call->outgoing ? "Outgoing" : "Incoming");
 
 	/* SDP debug */
 	err |= sdp_session_debug(pf, call->sdp);
@@ -909,8 +922,10 @@ int call_info(struct re_printf *pf, const struct call *call)
 	if (!call)
 		return 0;
 
-	return re_hprintf(pf, "%H  %8s  %s", print_duration, call,
-			  state_name(call->state), call->peer_uri);
+	return re_hprintf(pf, "%H  %9s  %s  %s", print_duration, call,
+			  state_name(call->state),
+			  call->on_hold ? "(on hold)" : "         ",
+			  call->peer_uri);
 }
 
 
@@ -1202,7 +1217,7 @@ static bool have_common_audio_codecs(const struct call *call)
 	if (!sc)
 		return false;
 
-	ac = sc->data;
+	ac = sc->data;  /* note: this will exclude telephone-event */
 
 	return ac != NULL;
 }
@@ -1217,6 +1232,8 @@ int call_accept(struct call *call, struct sipsess_sock *sess_sock,
 	if (!call || !msg)
 		return EINVAL;
 
+	call->outgoing = false;
+
 	got_offer = (mbuf_get_left(msg->mb) > 0);
 
 	err = pl_strdup(&call->peer_uri, &msg->from.auri);
@@ -1230,6 +1247,8 @@ int call_accept(struct call *call, struct sipsess_sock *sess_sock,
 	}
 
 	if (got_offer) {
+		struct sdp_media *m;
+		const struct sa *raddr;
 
 		err = sdp_decode(call->sdp, msg->mb, true);
 		if (err)
@@ -1237,6 +1256,30 @@ int call_accept(struct call *call, struct sipsess_sock *sess_sock,
 
 		call->got_offer = true;
 
+		/*
+		 * Each media description in the SDP answer MUST
+		 * use the same network type as the corresponding
+		 * media description in the offer.
+		 *
+		 * See RFC 6157
+		 */
+		m = stream_sdpmedia(audio_strm(call->audio));
+		raddr = sdp_media_raddr(m);
+
+		if (sa_af(raddr) != call->af) {
+			info("call: incompatible address-family"
+			     " (local=%s, remote=%s)\n",
+			     net_af2name(call->af),
+			     net_af2name(sa_af(raddr)));
+
+			sip_treply(NULL, uag_sip(), msg,
+				   488, "Not Acceptable Here");
+
+			call_event_handler(call, CALL_EVENT_CLOSED,
+					   "Wrong address family");
+			return 0;
+		}
+
 		/* Check if we have any common audio codecs, after
 		 * the SDP offer has been parsed
 		 */
@@ -1654,3 +1697,15 @@ void call_set_xrtpstat(struct call *call)
 				  "X-RTP-Stat: %H\r\n",
 				  audio_print_rtpstat, call->audio);
 }
+
+
+bool call_is_onhold(const struct call *call)
+{
+	return call ? call->on_hold : false;
+}
+
+
+bool call_is_outgoing(const struct call *call)
+{
+	return call ? call->outgoing : false;
+}
